import { describe, it, expect } from 'vitest'
import { Card } from '../card.js'
import { SM2Scheduler } from '../algorithms/sm2.js'

describe('SM2Scheduler', () => {
  it('debería establecer intervalo de 1 día en la primera revisión', () => {
    const card = new Card()
    const scheduler = new SM2Scheduler()
    const currentDate = new Date('2024-03-20T12:00:00.000Z')
    const quality = 3 // Calidad de la respuesta (0-5)

    scheduler.calculateNextInterval(card, quality, currentDate)

    expect(card.getLastReview()).toEqual(currentDate)
    expect(card.getNextReview()).toEqual(new Date('2024-03-21T12:00:00.000Z'))
    expect(card.getReviewCount()).toBe(1)
    expect(card.getMetadata().ef).toBe(2.5) // Factor de facilidad inicial
  })

  it('debería manejar correctamente la segunda revisión', () => {
    const card = new Card()
    const scheduler = new SM2Scheduler()
    const firstReviewDate = new Date('2024-03-20T12:00:00.000Z')
    const secondReviewDate = new Date('2024-03-21T12:00:00.000Z')

    // Primera revisión
    scheduler.calculateNextInterval(card, 3, firstReviewDate)
    
    // Segunda revisión con respuesta correcta
    scheduler.calculateNextInterval(card, 4, secondReviewDate)

    expect(card.getLastReview()).toEqual(secondReviewDate)
    expect(card.getNextReview()).toEqual(new Date('2024-03-27T12:00:00.000Z')) // 6 días después
    expect(card.getReviewCount()).toBe(2)
    // EF = 2.5 + (0.1 - (5-4) * (0.08 + (5-4) * 0.02))
    // EF = 2.5 + (0.1 - 1 * (0.08 + 1 * 0.02))
    // EF = 2.5 + (0.1 - 0.1)
    // EF = 2.5
    expect(card.getMetadata().ef).toBe(2.5)
  })

  it('debería reiniciar el intervalo si la respuesta es incorrecta en la segunda revisión', () => {
    const card = new Card()
    const scheduler = new SM2Scheduler()
    const firstReviewDate = new Date('2024-03-20T12:00:00.000Z')
    const secondReviewDate = new Date('2024-03-21T12:00:00.000Z')

    // Primera revisión
    scheduler.calculateNextInterval(card, 3, firstReviewDate)
    
    // Segunda revisión con respuesta incorrecta
    scheduler.calculateNextInterval(card, 2, secondReviewDate)

    expect(card.getLastReview()).toEqual(secondReviewDate)
    expect(card.getNextReview()).toEqual(new Date('2024-03-22T12:00:00.000Z')) // 1 día después
    expect(card.getReviewCount()).toBe(0) // Se reinicia el contador
    expect(card.getMetadata().ef).toBe(2.5) // El EF se mantiene igual
  })

  it('debería calcular el intervalo correctamente en la tercera revisión', () => {
    const card = new Card()
    const scheduler = new SM2Scheduler()
    const firstDate = new Date('2024-03-20T12:00:00.000Z')
    const secondDate = new Date('2024-03-21T12:00:00.000Z')
    const thirdDate = new Date('2024-03-27T12:00:00.000Z')

    // Primera revisión - intervalo 1 día
    scheduler.calculateNextInterval(card, 3, firstDate)
    
    // Segunda revisión - intervalo 6 días
    scheduler.calculateNextInterval(card, 4, secondDate)
    
    // Tercera revisión - intervalo previo * EF
    scheduler.calculateNextInterval(card, 5, thirdDate)

    expect(card.getLastReview()).toEqual(thirdDate)
    // 6 días * 2.6 = ~15 días (el EF aumentó por la calidad 5)
    expect(card.getNextReview()).toEqual(new Date('2024-04-11T12:00:00.000Z'))
    expect(card.getReviewCount()).toBe(3)
    // EF = 2.5 + (0.1 - (5-5) * (0.08 + (5-5) * 0.02))
    // EF = 2.5 + 0.1 = 2.6
    expect(card.getMetadata().ef).toBe(2.6)
  })
}) 