import { Scheduler } from '../scheduler.js'
import { Temporal } from '@js-temporal/polyfill'

export class SM2Scheduler extends Scheduler {
  calculateNextInterval(card, quality, currentDate) {
    const metadata = card.getMetadata()
    const reviewCount = card.getReviewCount()
    const getDateString = (date) => date.toISOString().split('T')[0]
    let currentPlainDate = Temporal.PlainDate.from(getDateString(currentDate))
    let ef = metadata.ef ?? 2.5 // Si no existe ef, inicializamos a 2.5

    if (quality >= 3) {
      // Solo actualizamos el EF cuando la calidad es 5
      if (quality === 5) {
        // Fórmula SM2: EF := EF + (0.1 - (5 - q) * (0.08 + (5 - q) * 0.02))
        const diffFrom5 = 5 - quality
        const multiplier = 0.08 + (diffFrom5 * 0.02)
        const adjustment = 0.1 - (diffFrom5 * multiplier)
        ef = Math.max(1.3, ef + adjustment)
      }

      if (reviewCount === 0) {
        // Primera revisión: intervalo de 1 día
        currentPlainDate = currentPlainDate.add({ days: 1 })
      } else if (reviewCount === 1) {
        // Segunda revisión: intervalo de 6 días
        currentPlainDate = currentPlainDate.add({ days: 6 })
      } else {
        // Tercera revisión en adelante: intervalo previo * EF
        const prevNextReview = Temporal.PlainDate.from(getDateString(card.getNextReview()))
        const prevLastReview = Temporal.PlainDate.from(getDateString(card.getLastReview()))
        const prevInterval = prevNextReview.since(prevLastReview).days
        const nextInterval = Math.floor(prevInterval * ef) // Redondeamos hacia abajo
        currentPlainDate = currentPlainDate.add({ days: nextInterval })
      }
    } else {
      // Respuesta incorrecta: reiniciar a 1 día
      currentPlainDate = currentPlainDate.add({ days: 1 })
      if (reviewCount > 0) {
        // Solo reiniciamos el contador si no es la primera revisión
        card.updateReview(quality, currentDate, new Date(currentPlainDate.toString() + 'T12:00:00.000Z'), { ef: 2.5 }, true)
        return
      }
    }

    // Actualizamos la tarjeta con el nuevo estado
    card.updateReview(
      quality,
      currentDate,
      new Date(currentPlainDate.toString() + 'T12:00:00.000Z'),
      { ef }
    )
  }
} 